/*-------------------------------------------------------------------------
 * Copyright (c) 1999-2001 Kenneth W. Sodemann (stufflehead@bigfoot.com)
 *-------------------------------------------------------------------------
 * attrib_dlg_db
 *
 * Synopsis:
 *   Routines to simplify DB access for the attributes data.
 *
 * $Id: attrib_dlg_db.c,v 1.11 2001/11/17 22:36:50 stuffle Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to
 * Free Software Foundation, Inc.
 * 59 Temple Place, Suite 330
 * Boston, MA  02111-1307  USA
 *
 *-------------------------------------------------------------------------
 */
#include "error_chks.h"
#include "attrib_dlg_db.h"
#include <ctype.h>
#include <libpq-fe.h>
#include <stdlib.h>
#include <string.h>

#define PK_NUM_POS      0
#define NAME_POS        1
#define RANK_POS        2
#define ACTIVE_POS      3
#define DESCR_POS       4
#define DEF_SEVR_POS    5

#define BUFFER_PADDING  200

/*------------------------------------------------------------------------
 * read_xxxx_data functions
 *------------------------------------------------------------------------ 
 */
PGresult *
read_sevr_data (PGconn *conn, gboolean no_inactive)
{
   char buffer[BUFFER_PADDING];

   strcpy (buffer, "SELECT severity_num, name, ");
   strcat (buffer, "order_num, active, descr ");
   strcat (buffer, "FROM severity ");
   if (no_inactive)
      {
      strcat (buffer, "WHERE active = TRUE ");
      }
   strcat (buffer, "ORDER BY order_num ");

   return PQexec (conn, buffer);
}

PGresult *
read_stat_data (PGconn *conn, gboolean no_inactive)
{
   char buffer[BUFFER_PADDING];
   
   strcpy (buffer, "SELECT status_num, name, ");
   strcat (buffer, "order_num, active, descr ");
   strcat (buffer, "FROM status ");
   if (no_inactive)
      {
      strcat (buffer, "WHERE active = TRUE ");
      }
   strcat (buffer, "ORDER BY order_num ");
   
   return PQexec (conn, buffer);
}

PGresult *
read_type_data (PGconn *conn, gboolean no_inactive)
{
   char buffer[BUFFER_PADDING];
   
   strcpy (buffer, "SELECT t.type_num, t.name, s.order_num, ");
   strcat (buffer, "t.active, t.descr, s.name ");
   strcat (buffer, "FROM problem_type t, severity s ");
   strcat (buffer, "WHERE s.severity_num = t.severity_num ");
   if (no_inactive)
      {
      strcat (buffer, "AND t.active = TRUE ");
      }
   strcat (buffer, "ORDER BY t.order_num, t.name ");
   
   return PQexec (conn, buffer);
}



PGresult *
delete_severity (PGconn  *conn,
                 long     pk_num)
{
   char sql_buffer[BUFFER_PADDING];
   
   sprintf (sql_buffer, 
            "DELETE FROM severity WHERE severity_num = %ld",
            pk_num);
   
   return PQexec (conn, sql_buffer);
}
  

PGresult *
delete_status (PGconn  *conn,
               long     pk_num)
{
   char sql_buffer[BUFFER_PADDING];
   
   sprintf (sql_buffer, 
            "DELETE FROM status WHERE status_num = %ld",
            pk_num);
   
   return PQexec (conn, sql_buffer);
}


PGresult *
delete_type (PGconn  *conn,
             long     pk_num)
{
   char sql_buffer[BUFFER_PADDING];
   
   sprintf (sql_buffer, 
            "DELETE FROM problem_type WHERE type_num = %ld",
            pk_num);
   
   return PQexec (conn, sql_buffer);
}


/*------------------------------------------------------------------------
 * get_xxxx functions
 *------------------------------------------------------------------------ 
 */
const char *
get_pk_num (PGresult *res, int n)
{
   if (n >= PQntuples (res))
      {
      return "";
      }
   
   return PQgetvalue (res, n, PK_NUM_POS);
}

const char *
get_name (PGresult *res, int n)
{
   if (n >= PQntuples (res)) 
      {
      return "";
      }

   return PQgetvalue (res, n, NAME_POS);
}

const char *
get_rank (PGresult *res, int n)
{
   if (n >= PQntuples (res)) 
      {
      return "";
      }

   return PQgetvalue (res, n, RANK_POS);
}

const char *
get_active_yn (PGresult *res, int n)
{
   if (n >= PQntuples (res)) 
      {
      return "";
      }

   if ((PQgetvalue (res, n, ACTIVE_POS))[0] == 't')
      {
      return "Yes";
      }
   
   return "No";
}

const char *
get_descr (PGresult *res, int n)
{
   if (n >= PQntuples (res))
      {
      return "";
      }
   
   return PQgetvalue (res, n, DESCR_POS);
}


const char *
get_def_sevr (PGresult *res, int n)
{
   if (n >= PQntuples (res))
      {
      return "";
      }
   
   return PQgetvalue (res, n, DEF_SEVR_POS);
}
