/*-------------------------------------------------------------------------
 * Copyright (c) 1999-2002 Kenneth W. Sodemann (stufflehead@bigfoot.com)
 *-------------------------------------------------------------------------
 * person_dlg
 *
 * Synopsis:
 *   Handle the user list dialog and the user editor.
 *
 * $Id: person_dlg.c,v 1.39 2002/01/05 17:43:51 stuffle Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to
 * Free Software Foundation, Inc.
 * 59 Temple Place, Suite 330
 * Boston, MA  02111-1307  USA
 *
 *-------------------------------------------------------------------------
 */
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include <gnome.h>
#include <libpq-fe.h>

#include <preps/gstr_utils.h>

#include "db_utils.h"
#include "defs.h"
#include "error_chks.h"
#include "gtkutils.h"
#include "prepsmain.h"
#include "util.h"

/*
 *  Define the names of the significant widgets
 */
#define PERSON_DIALOG       "person_dlg"
#define PERSON_LIST         "person_clist"
#define ADD_BUTTON          "add_button"
#define MOD_BUTTON          "modify_button"
#define DEL_BUTTON          "delete_button"
#define DISMISS_BUTTON      "dismiss_button"
#define OK_BUTTON           "ok_button"
#define CANCEL_BUTTON       "cancel_button"
#define SHOW_INACTIVE_CB    "show_inactive_cb"
#define PERSON_EDIT_DIALOG  "person_edit_dlg"
#define LAST_NAME_ENTRY     "last_name_entry"
#define FIRST_NAME_ENTRY    "first_name_entry"
#define MIDDLE_NAME_ENTRY   "middle_name_entry"
#define EMAIL_ENTRY         "email_address_entry"
#define LOGIN_ID_ENTRY      "login_id_entry"
#define USER_RB             "user_access_rb"
#define ADMIN_RB            "admin_access_rb"
#define ACTIVE_CB           "active_cb"

#define LOGIN_ID_COL        2

#define USER_ACCESS         0
#define ADMIN_ACCESS        1

/*
 *  This is the order in the main SELECT.  If the order changes
 *  in 'read_person_data', these must change to match it.
 *  
 *  These defines also tie in to the order of the SELECT that follows
 *  them (which is used for the modify dialog).
 */
#define LOGIN_ID_POS     0
#define LAST_NAME_POS    1
#define FIRST_NAME_POS   2
#define ACTIVE_POS       3
#define ACCESS_FLAG_POS  4
#define MIDDLE_NAME_POS  5
#define E_MAIL_POS       6 
#define PERSON_QUERY "SELECT login_id, last_name, first_name, active, \
                             access_flag, middle_name, e_mail \
                        FROM person \
                       WHERE person.login_id = '%s' "
                       

static const gchar *my_child_list = "CHILD_LIST";

/*
 * The single allowed person list dialog box.
 */
static GtkWidget  *person_dlg = NULL;

/*
 *  Local prototypes
 */
static gboolean draw_person_list (GtkWindow *parent, PGconn *conn);


static void 
on_person_edit_dlg_show (GtkWidget *widget, gpointer user_data)
{
   GString      *query;
   PGresult     *result;
   GtkWidget    *child;
   gboolean      query_error = FALSE;
   gboolean      is_update;
   gchar        *login_id;
   PGconn       *conn;

   g_assert (user_data == NULL);

   is_update = (GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget),
                                                      IS_UPDATE)) == 1);
   conn = gtk_object_get_data (GTK_OBJECT (widget), MY_DB_CONN);
   g_assert (conn != NULL);

   if (is_update)
      {
      login_id = gtk_object_get_data (GTK_OBJECT (widget), LOGIN_ID);
      g_assert (login_id != NULL);
   
      query = g_string_new ("");
      g_string_sprintf (query, PERSON_QUERY, login_id);

      result = PQexec (conn, query->str);
      if (chk_sql_error (result, _("getting user data to edit")))
         {
         child = get_widget (widget, LAST_NAME_ENTRY);
         g_assert (child != NULL);
         gtk_entry_set_text (GTK_ENTRY (child),
               PQgetvalue (result, 0, LAST_NAME_POS));
         
         child = get_widget (widget, FIRST_NAME_ENTRY);
         g_assert (child != NULL);
         gtk_entry_set_text (GTK_ENTRY (child),
               PQgetvalue (result, 0, FIRST_NAME_POS));
         
         child = get_widget (widget, MIDDLE_NAME_ENTRY);
         g_assert (child != NULL);
         gtk_entry_set_text (GTK_ENTRY (child),
               PQgetvalue (result, 0, MIDDLE_NAME_POS));
         
         child = get_widget (widget, EMAIL_ENTRY);
         g_assert (child != NULL);
         gtk_entry_set_text (GTK_ENTRY (child),
               PQgetvalue (result, 0, E_MAIL_POS));
         
         child = get_widget (widget, LOGIN_ID_ENTRY);
         g_assert (child != NULL);
         gtk_entry_set_text (GTK_ENTRY (child),
               PQgetvalue (result, 0, LOGIN_ID_POS));
         
         child = get_widget (widget, ACTIVE_CB);
         g_assert (child != NULL);
         if ((PQgetvalue (result, 0, ACTIVE_POS))[0] == 't')
            {
            gtk_toggle_button_set_active
               (GTK_TOGGLE_BUTTON (child), TRUE);
            }
         else
            {
            gtk_toggle_button_set_active
               (GTK_TOGGLE_BUTTON (child), FALSE);
            }
         
         switch (atoi (PQgetvalue (result, 0, ACCESS_FLAG_POS)))
            {
            case 0:
               child = get_widget (widget, USER_RB);
               break;
            case 1:
               child = get_widget (widget, ADMIN_RB);
               break;
            default:
               g_assert_not_reached ();
            }
         g_assert (child != NULL);
         gtk_toggle_button_set_active
            (GTK_TOGGLE_BUTTON (child), TRUE);

         }
      else /* chk_sql_error returned FALSE */
         {
         query_error = TRUE;
         }
      g_string_free (query, TRUE);
      PQclear (result);
      }
   
   if (query_error)
      {
      gnome_dialog_close (GNOME_DIALOG (widget));
      }
   else
      {
      set_all_clean (widget);
      }
}


static void 
on_person_edit_dlg_destroy (GtkObject  *object, gpointer  user_data)
{
   destroy_dirty_list (GTK_WIDGET (object));
}


static GString *
build_insert_string (GnomeDialog *dlg)
{
   GtkWidget        *widget;
   GtkRadioButton   *user_rb;
   GtkRadioButton   *admin_rb;
   GString          *str;
   GString          *tmp;
   gint              access_cd = USER_ACCESS;

   tmp = g_string_new ("");
   str = g_string_new ("INSERT INTO person (login_id, last_name, ");
   str = g_string_append (str, "first_name, middle_name, e_mail, ");
   str = g_string_append (str, "active, access_flag) VALUES (");

   widget = get_widget (GTK_WIDGET (dlg), LOGIN_ID_ENTRY);
   g_assert (widget != NULL);
   tmp = g_string_assign (tmp, gtk_entry_get_text (GTK_ENTRY (widget)));
   g_string_prepare_db_instr (tmp);
   str = g_string_append (str, tmp->str);
   str = g_string_append (str, ", ");

   widget = get_widget (GTK_WIDGET (dlg), LAST_NAME_ENTRY);
   g_assert (widget != NULL);
   tmp = g_string_assign (tmp, gtk_entry_get_text (GTK_ENTRY (widget)));
   g_string_prepare_db_instr (tmp);
   str = g_string_append (str, tmp->str);
   str = g_string_append (str, ", ");

   widget = get_widget (GTK_WIDGET (dlg), FIRST_NAME_ENTRY);
   g_assert (widget != NULL);
   tmp = g_string_assign (tmp, gtk_entry_get_text (GTK_ENTRY (widget)));
   g_string_prepare_db_instr (tmp);
   str = g_string_append (str, tmp->str);
   str = g_string_append (str, ", ");

   widget = get_widget (GTK_WIDGET (dlg), MIDDLE_NAME_ENTRY);
   g_assert (widget != NULL);
   tmp = g_string_assign (tmp, gtk_entry_get_text (GTK_ENTRY (widget)));
   g_string_prepare_db_instr (tmp);
   str = g_string_append (str, tmp->str);
   str = g_string_append (str, ", ");

   widget = get_widget (GTK_WIDGET (dlg), EMAIL_ENTRY);
   g_assert (widget != NULL);
   tmp = g_string_assign (tmp, gtk_entry_get_text (GTK_ENTRY (widget)));
   g_string_prepare_db_instr (tmp);
   str = g_string_append (str, tmp->str);
   str = g_string_append (str, ", ");

   widget = get_widget (GTK_WIDGET (dlg), ACTIVE_CB);
   g_assert (widget != NULL);
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
      {
      str = g_string_append (str, "TRUE, ");
      }
   else
      {
      str = g_string_append (str, "FALSE, ");
      }

   user_rb = GTK_RADIO_BUTTON (get_widget (GTK_WIDGET (dlg), USER_RB));
   admin_rb = GTK_RADIO_BUTTON (get_widget (GTK_WIDGET (dlg), ADMIN_RB));
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (user_rb)))
      {
      access_cd = USER_ACCESS;
      }
   else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (admin_rb)))
      {
      access_cd = ADMIN_ACCESS;
      }
   else
      {
      g_assert_not_reached ();
      }
   g_string_sprintf (tmp, "%d", access_cd);
   str = g_string_append (str, tmp->str);
   str = g_string_append (str, ")");

   g_string_free (tmp, TRUE);

   return str;
}

static GString *
build_update_string (GnomeDialog *dlg)
{
   GtkWidget        *widget;
   GtkRadioButton   *user_rb;
   GtkRadioButton   *admin_rb;
   GString          *str;
   GString          *tmp;
   gboolean          first = TRUE;
   gint              access_cd = USER_ACCESS;

   str = g_string_new ("UPDATE person SET ");
   tmp = g_string_new ("");

   widget = get_widget (GTK_WIDGET (dlg), LAST_NAME_ENTRY);
   g_assert (widget != NULL);
   if (widget_is_dirty (widget))
      {
      tmp = g_string_assign (tmp, gtk_entry_get_text (GTK_ENTRY (widget)));
      g_string_prepare_db_instr (tmp);
      if (!first)
         {
         str = g_string_append (str, ", ");
         }
      first = FALSE;
      str = g_string_append (str, "last_name = ");
      str = g_string_append (str, tmp->str);
      }

   widget = get_widget (GTK_WIDGET (dlg), FIRST_NAME_ENTRY);
   g_assert (widget != NULL);
   if (widget_is_dirty (widget))
      {
      tmp = g_string_assign (tmp, gtk_entry_get_text (GTK_ENTRY (widget)));
      g_string_prepare_db_instr (tmp);
      if (!first)
         {
         str = g_string_append (str, ", ");
         }
      first = FALSE;
      str = g_string_append (str, "first_name = ");
      str = g_string_append (str, tmp->str);
      }

   widget = get_widget (GTK_WIDGET (dlg), MIDDLE_NAME_ENTRY);
   g_assert (widget != NULL);
   if (widget_is_dirty (widget))
      {
      tmp = g_string_assign (tmp, gtk_entry_get_text (GTK_ENTRY (widget)));
      g_string_prepare_db_instr (tmp);
      if (!first)
         {
         str = g_string_append (str, ", ");
         }
      first = FALSE;
      str = g_string_append (str, "middle_name = ");
      str = g_string_append (str, tmp->str);
      }
   
   widget = get_widget (GTK_WIDGET (dlg), EMAIL_ENTRY);
   g_assert (widget != NULL);
   if (widget_is_dirty (widget))
      {
      tmp = g_string_assign (tmp, gtk_entry_get_text (GTK_ENTRY (widget)));
      g_string_prepare_db_instr (tmp);
      if (!first)
         {
         str = g_string_append (str, ", ");
         }
      first = FALSE;
      str = g_string_append (str, "e_mail = ");
      str = g_string_append (str, tmp->str);
      }

   widget = get_widget (GTK_WIDGET (dlg), ACTIVE_CB);
   g_assert (widget != NULL);
   if (widget_is_dirty (widget))
      {
      if (!first)
         {
         str = g_string_append (str, ", ");
         }
      first = FALSE;
      str = g_string_append (str, "active = ");
      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
         {
         str = g_string_append (str, "TRUE");
         }
      else
         {
         str = g_string_append (str, "FALSE");
         }
      }
   
   user_rb = GTK_RADIO_BUTTON (get_widget (GTK_WIDGET (dlg), USER_RB));
   admin_rb = GTK_RADIO_BUTTON (get_widget (GTK_WIDGET (dlg), ADMIN_RB));
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (user_rb)))
      {
      access_cd = USER_ACCESS;
      }
   else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (admin_rb)))
      {
      access_cd = ADMIN_ACCESS;
      }
   else
      {
      g_assert_not_reached ();
      }

   if (widget_is_dirty (GTK_WIDGET (user_rb)))
      {
      g_string_sprintf (tmp, "%d", access_cd);
      if (!first)
         {
         str = g_string_append (str, ", ");
         }
      first = FALSE;
      str = g_string_append (str, "access_flag = ");
      str = g_string_append (str, tmp->str);
      }

   widget = get_widget (GTK_WIDGET (dlg), LOGIN_ID_ENTRY);
   g_assert (widget != NULL);
   tmp = g_string_assign (tmp, gtk_entry_get_text (GTK_ENTRY (widget)));
   g_string_prepare_db_instr (tmp);
   str = g_string_append (str, " WHERE login_id = ");
   str = g_string_append (str, tmp->str);

   g_string_free (tmp, TRUE);

   return str;
}


static void 
on_ok_button_clicked (GnomeDialog *dlg)
{
   PGresult           *res;
   gboolean            in_error = FALSE;
   PGconn             *conn;
   gboolean            is_update;
   GtkWidget          *parent;
   GString            *sqlstr;
   
   conn = gtk_object_get_data (GTK_OBJECT (dlg), MY_DB_CONN);
   g_assert (conn != NULL);
   parent = gtk_object_get_data (GTK_OBJECT (dlg), MY_PARENT);
   g_assert (parent != NULL);
   is_update = (GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (dlg),
                                                      IS_UPDATE)) == 1);

   START_TRANSACTION (conn);
   if (is_update)
      {
      if (widgets_children_are_dirty (GTK_WIDGET (dlg)))
         {
         sqlstr = build_update_string (dlg);
         res = PQexec (conn, sqlstr->str);
         in_error = !chk_sql_error (res, _("updating user data"));
         g_string_free (sqlstr, TRUE);
         PQclear (res);
         }
      }
   else
      {
      sqlstr = build_insert_string (dlg);
      res = PQexec (conn, sqlstr->str);
      in_error = !chk_sql_error (res, _("inserting user data"));
      g_string_free (sqlstr, TRUE);
      PQclear (res);
      }

   if (!in_error)
      {
      COMMIT (conn);
      draw_person_list (GTK_WINDOW (parent), conn);
      gnome_dialog_close (dlg);
      }
   else
      {
      ROLLBACK (conn);
      }
}


static void 
on_cancel_button_clicked (GnomeDialog *dlg)
{
   gnome_dialog_close (dlg);
}


static void
on_person_edit_dlg_clicked (GnomeDialog  *dlg,
                            gint          btn_cd,
                            gpointer      user_data)
{
   switch (btn_cd)
      {
      case 0:
         on_ok_button_clicked (dlg);
         break;
      case 1:
         on_cancel_button_clicked (dlg);
         break;
      case -1:
         break;
      default:
         g_assert_not_reached();
      }
}


static GtkWidget* 
create_person_edit_dlg (gboolean     is_update,
                        gchar       *login_id,
                        PGconn      *conn,
                        GtkWidget   *parent,
                        gboolean     user_is_admin)
{
  GtkWidget           *person_edit_dlg;
  GtkWidget           *dialog_vbox9;
  GtkWidget           *table3;
  GtkWidget           *label5;
  GtkWidget           *last_name_entry;
  GtkWidget           *login_id_entry;
  GtkWidget           *email_address_entry;
  GtkWidget           *frame3;
  GtkWidget           *vbox21;
  GSList              *vbox21_group = NULL;
  GtkWidget           *user_access_rb;
  GtkWidget           *admin_access_rb;
  GtkWidget           *label3;
  GtkWidget           *label1;
  GtkWidget           *label2;
  GtkWidget           *first_name_entry;
  GtkWidget           *middle_name_entry;
  GtkWidget           *label4;
  GtkWidget           *active_cb;
  GtkWidget           *popup;
  gchar               *tmp_id;
  GtkTooltips         *tooltips;
  
  tooltips = gtk_object_get_data (GTK_OBJECT (parent), TOOLTIPS);
  g_assert (tooltips != NULL);

  if (is_update)
     {
     person_edit_dlg = gnome_dialog_new (_("Update User"),
                                         GNOME_STOCK_BUTTON_OK,
                                         GNOME_STOCK_BUTTON_CANCEL,
                                         NULL);
     }
  else
     {
     person_edit_dlg = gnome_dialog_new (_("Add User"),
                                         GNOME_STOCK_BUTTON_OK,
                                         GNOME_STOCK_BUTTON_CANCEL,
                                         NULL);
     }
  gnome_dialog_set_parent (GNOME_DIALOG (person_edit_dlg), GTK_WINDOW (parent));
  gtk_object_set_data (GTK_OBJECT (person_edit_dlg), 
                       PERSON_EDIT_DIALOG, 
                       person_edit_dlg);
  gtk_container_border_width (GTK_CONTAINER (person_edit_dlg), 5);
  gtk_signal_connect (GTK_OBJECT (person_edit_dlg), "show",
                      GTK_SIGNAL_FUNC (on_person_edit_dlg_show),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (person_edit_dlg), "destroy",
                      GTK_SIGNAL_FUNC (on_person_edit_dlg_destroy),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (person_edit_dlg), "clicked",
                      GTK_SIGNAL_FUNC (on_person_edit_dlg_clicked),
                      NULL);
  gnome_dialog_set_close (GNOME_DIALOG (person_edit_dlg), FALSE);
  gtk_window_set_policy (GTK_WINDOW (person_edit_dlg), FALSE, FALSE, TRUE);
  if (user_is_admin)
     {
     gnome_dialog_set_default (GNOME_DIALOG (person_edit_dlg), 0);
     }
  else
     {
     gnome_dialog_set_default (GNOME_DIALOG (person_edit_dlg), 1);
     gnome_dialog_set_sensitive (GNOME_DIALOG (person_edit_dlg), 0, FALSE);
     }

  gtk_object_set_data (GTK_OBJECT (person_edit_dlg), TOP_DOG, person_edit_dlg);
  gtk_object_set_data (GTK_OBJECT (person_edit_dlg), MY_PARENT, parent);
  gtk_object_set_data (GTK_OBJECT (person_edit_dlg), MY_DB_CONN, conn);
  gtk_object_set_data (GTK_OBJECT (person_edit_dlg), IS_UPDATE,
                       GINT_TO_POINTER (is_update ? 1 : 0));
  tmp_id = g_malloc (strlen (login_id) + 1);
  strcpy (tmp_id, login_id);
  gtk_object_set_data_full (GTK_OBJECT (person_edit_dlg),
                            LOGIN_ID,
                            tmp_id,
                            g_free);

  dialog_vbox9 = GNOME_DIALOG (person_edit_dlg)->vbox;
  gtk_object_set_data (GTK_OBJECT (person_edit_dlg), 
                       "dialog_vbox9", 
                       dialog_vbox9);
  gtk_box_set_spacing (GTK_BOX (dialog_vbox9), 5);
  gtk_widget_show (dialog_vbox9);

  table3 = gtk_table_new (4, 4, FALSE);
  gtk_object_set_data (GTK_OBJECT (person_edit_dlg), 
                       "table3", table3);
  gtk_widget_show (table3);
  gtk_box_pack_start (GTK_BOX (dialog_vbox9), table3, TRUE, TRUE, 0);
  gtk_table_set_row_spacings (GTK_TABLE (table3), 3);
  gtk_table_set_col_spacings (GTK_TABLE (table3), 3);

  label5 = gtk_label_new (_("E-Mail Address:"));
  gtk_object_set_data (GTK_OBJECT (person_edit_dlg), "label5", label5);
  gtk_widget_show (label5);
  gtk_table_attach (GTK_TABLE (table3), 
                    label5, 2, 3, 2, 3,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label5), 0, 0.5);

  last_name_entry = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (person_edit_dlg), 
                       LAST_NAME_ENTRY, 
                       last_name_entry);
  gtk_widget_show (last_name_entry);
  gtk_table_attach (GTK_TABLE (table3), last_name_entry, 1, 2, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_tooltips_set_tip (tooltips, 
                        last_name_entry, 
                        _("Last name is required."), 
                        NULL);
  gtk_widget_set_sensitive (last_name_entry, user_is_admin);
  gtk_signal_connect (GTK_OBJECT (last_name_entry), "changed",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  add_widget_to_dirty_list (person_edit_dlg, last_name_entry);
  popup = attach_generic_edit_popup_menu (last_name_entry);
  gtk_widget_ref (popup);
  gtk_object_set_data_full (GTK_OBJECT (person_edit_dlg), "popup1", popup,
                            (GtkDestroyNotify) gtk_widget_unref);

  login_id_entry = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (person_edit_dlg), 
                       LOGIN_ID_ENTRY, 
                       login_id_entry);
  gtk_widget_show (login_id_entry);
  gtk_table_attach (GTK_TABLE (table3), login_id_entry, 1, 2, 2, 3,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_tooltips_set_tip (tooltips, login_id_entry, 
    _("This user's PostgreSQL login ID (usually matches the system login ID)."),
    NULL);
  /* 
   * We don't want the user to mess with the login ID if this
   * is an update.
   */
  gtk_entry_set_editable (GTK_ENTRY (login_id_entry), !is_update);
  gtk_widget_set_sensitive (login_id_entry, user_is_admin && !is_update);
  popup = attach_generic_edit_popup_menu (login_id_entry);
  gtk_widget_ref (popup);
  gtk_object_set_data_full (GTK_OBJECT (person_edit_dlg), "popup2", popup,
                            (GtkDestroyNotify) gtk_widget_unref);

  email_address_entry = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (person_edit_dlg),
                       EMAIL_ENTRY, 
                       email_address_entry);
  gtk_widget_show (email_address_entry);
  gtk_table_attach (GTK_TABLE (table3), email_address_entry, 3, 4, 2, 3,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_tooltips_set_tip (tooltips, email_address_entry, 
        _("Optional e-mail address must be fully qualified if not local."), 
        NULL);
  gtk_widget_set_sensitive (email_address_entry, user_is_admin);
  gtk_signal_connect (GTK_OBJECT (email_address_entry), "changed",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  add_widget_to_dirty_list (person_edit_dlg, email_address_entry);
  popup = attach_generic_edit_popup_menu (email_address_entry);
  gtk_widget_ref (popup);
  gtk_object_set_data_full (GTK_OBJECT (person_edit_dlg), "popup3", popup,
                            (GtkDestroyNotify) gtk_widget_unref);

  frame3 = gtk_frame_new (_("Access"));
  gtk_object_set_data (GTK_OBJECT (person_edit_dlg), "frame3", frame3);
  gtk_widget_show (frame3);
  gtk_table_attach (GTK_TABLE (table3), frame3, 0, 2, 3, 4,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);

  vbox21 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (person_edit_dlg), "vbox21", vbox21);
  gtk_widget_show (vbox21);
  gtk_container_add (GTK_CONTAINER (frame3), vbox21);

  user_access_rb = gtk_radio_button_new_with_label (vbox21_group, _("User"));
  vbox21_group = gtk_radio_button_group (GTK_RADIO_BUTTON (user_access_rb));
  gtk_object_set_data (GTK_OBJECT (person_edit_dlg), 
                       USER_RB, 
                       user_access_rb);
  gtk_widget_show (user_access_rb);
  gtk_box_pack_start (GTK_BOX (vbox21), user_access_rb, TRUE, TRUE, 0);
  gtk_widget_set_sensitive (user_access_rb, user_is_admin);
  /*
   * With radio buttons, if one in a group is dirty, they are all dirty,
   * so just do this with the "User" radio button.
   */
  gtk_signal_connect (GTK_OBJECT (user_access_rb), "toggled",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  add_widget_to_dirty_list (person_edit_dlg, user_access_rb);

  admin_access_rb = gtk_radio_button_new_with_label (vbox21_group, 
                                                     _("Administrator"));
  vbox21_group = gtk_radio_button_group (GTK_RADIO_BUTTON (admin_access_rb));
  gtk_object_set_data (GTK_OBJECT (person_edit_dlg), 
                       ADMIN_RB, 
                       admin_access_rb);
  gtk_widget_show (admin_access_rb);
  gtk_box_pack_start (GTK_BOX (vbox21), admin_access_rb, TRUE, TRUE, 0);
  gtk_widget_set_sensitive (admin_access_rb, user_is_admin);

  label3 = gtk_label_new (_("Last Name:"));
  gtk_object_set_data (GTK_OBJECT (person_edit_dlg), "label3", label3);
  gtk_widget_show (label3);
  gtk_table_attach (GTK_TABLE (table3), label3, 0, 1, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label3), 0, 0.5);

  label1 = gtk_label_new (_("Login ID:"));
  gtk_object_set_data (GTK_OBJECT (person_edit_dlg), "label1", label1);
  gtk_widget_show (label1);
  gtk_table_attach (GTK_TABLE (table3), label1, 0, 1, 2, 3,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);

  label2 = gtk_label_new (_("First Name:"));
  gtk_object_set_data (GTK_OBJECT (person_edit_dlg), "label2", label2);
  gtk_widget_show (label2);
  gtk_table_attach (GTK_TABLE (table3), label2, 2, 3, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);

  first_name_entry = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (person_edit_dlg), 
                       FIRST_NAME_ENTRY, 
                       first_name_entry);
  gtk_widget_show (first_name_entry);
  gtk_table_attach (GTK_TABLE (table3), first_name_entry, 3, 4, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_tooltips_set_tip (tooltips, 
                        first_name_entry, 
                        _("First name is required."), 
                        NULL);
  gtk_widget_set_sensitive (first_name_entry, user_is_admin);
  gtk_signal_connect (GTK_OBJECT (first_name_entry), "changed",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  add_widget_to_dirty_list (person_edit_dlg, first_name_entry);
  popup = attach_generic_edit_popup_menu (first_name_entry);
  gtk_widget_ref (popup);
  gtk_object_set_data_full (GTK_OBJECT (person_edit_dlg), "popup4", popup,
                            (GtkDestroyNotify) gtk_widget_unref);

  middle_name_entry = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (person_edit_dlg), 
                       MIDDLE_NAME_ENTRY, 
                       middle_name_entry);
  gtk_widget_show (middle_name_entry);
  gtk_table_attach (GTK_TABLE (table3), middle_name_entry, 1, 2, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_tooltips_set_tip (tooltips, 
                        middle_name_entry, 
                        _("Middle name (or initial) is optional."), 
                        NULL);
  gtk_widget_set_sensitive (middle_name_entry, user_is_admin);
  gtk_signal_connect (GTK_OBJECT (middle_name_entry), "changed",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  add_widget_to_dirty_list (person_edit_dlg, middle_name_entry);
  popup = attach_generic_edit_popup_menu (middle_name_entry);
  gtk_widget_ref (popup);
  gtk_object_set_data_full (GTK_OBJECT (person_edit_dlg), "popup5", popup,
                            (GtkDestroyNotify) gtk_widget_unref);

  label4 = gtk_label_new (_("Middle Name:"));
  gtk_object_set_data (GTK_OBJECT (person_edit_dlg), "label4", label4);
  gtk_widget_show (label4);
  gtk_table_attach (GTK_TABLE (table3), label4, 0, 1, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label4), 0, 0.5);

  active_cb = gtk_check_button_new_with_label (_("Active"));
  gtk_object_set_data (GTK_OBJECT (person_edit_dlg), "active_cb", active_cb);
  gtk_widget_show (active_cb);
  gtk_table_attach (GTK_TABLE (table3), active_cb, 2, 3, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (active_cb), TRUE);
  gtk_widget_set_sensitive (active_cb, user_is_admin);
  gtk_signal_connect (GTK_OBJECT (active_cb), "toggled",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  add_widget_to_dirty_list (person_edit_dlg, active_cb);

  gtk_object_set_data (GTK_OBJECT (person_edit_dlg), TOOLTIPS, tooltips);

  return person_edit_dlg;
}

/*--------------------------------------------------------------------------
 *  Local support functions for the person_dlg dialog.
 *  
 *  read_person_data : read the person data from the database
 *  draw_person_list : draw the person list     
 *--------------------------------------------------------------------------
 */
static PGresult *
read_person_data (PGconn *conn, gboolean show_inactive)
{
   GString  *buffer;
   PGresult *result;
   
   /*
    *  The order in the following SELECT is tied to #define's at 
    *  the start of the file.  Change this, and be sure to 
    *  change those to match.
    */
   buffer = g_string_new ("SELECT login_id, last_name, first_name, ");
   buffer = g_string_append (buffer, "active, access_flag ");
   buffer = g_string_append (buffer, "FROM person ");
   if (!show_inactive)
      {
      buffer = g_string_append (buffer, "WHERE active = TRUE ");
      }
   buffer = g_string_append (buffer, "ORDER BY 2, 3, 1 ");
   
   result = PQexec (conn, buffer->str);
   
   g_string_free (buffer, TRUE);
   return result;
}


static PGresult *
delete_person (PGconn *conn, const gchar *login_id)
{
   GString   *buffer;
   PGresult  *res;
   
   buffer = g_string_new ("");
   g_string_sprintf (buffer, 
                     "DELETE FROM person WHERE login_id = '%s'",
                     login_id);
   res = PQexec (conn, buffer->str);
   g_string_free (buffer, TRUE);
   
   return res;
}
   
   
static const char *
get_active_yn (PGresult *res, int n)
{
   if (n >= PQntuples (res))
      {
      return "";
      }

   if ((PQgetvalue (res, n, ACTIVE_POS))[0] == 't')
      {
      return "Yes";
      }

   return "No";
}


static const char *
get_access (PGresult *res, int n)
{
   if (n >= PQntuples (res))
      {
      return "";
      }
   
   if (atoi (PQgetvalue (res, n, ACCESS_FLAG_POS)) == 1)
      {
      return "Admin";
      }
   
   return "User";
}



static gboolean 
draw_person_list (GtkWindow *parent, PGconn *conn)
{
   char           *buffer[5] = {NULL, NULL, NULL, NULL, NULL};
   gint            ntuples;
   gint            i;
   PGresult       *res;
   GtkCList       *lb_widget;
   GtkCheckButton *cb_widget;
   gboolean        show_inactive;
   gboolean        rtn_val = FALSE;

   lb_widget = GTK_CLIST (get_widget (GTK_WIDGET(parent), PERSON_LIST));
   g_assert (lb_widget != NULL);
   cb_widget = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (parent),
                                             SHOW_INACTIVE_CB));
   g_assert (cb_widget != NULL);
   show_inactive = gtk_toggle_button_get_active
      (GTK_TOGGLE_BUTTON (cb_widget));
   
   gtk_clist_clear (lb_widget);
   res = read_person_data (conn, show_inactive);
   
   if (chk_sql_error (res, _("reading person data")))
      {
      ntuples = PQntuples (res);
      
      for (i = 0; i < ntuples; i++)
         {
         buffer[0] = (char *)g_malloc
            (strlen (PQgetvalue (res, i, LAST_NAME_POS)) + 1);
         strcpy (buffer[0], PQgetvalue (res, i, LAST_NAME_POS));
         buffer[1] = (char *)g_malloc
            (strlen (PQgetvalue (res, i, FIRST_NAME_POS)) + 1);
         strcpy (buffer[1], PQgetvalue (res, i, FIRST_NAME_POS));
         buffer[2] = (char *)g_malloc 
            (strlen (PQgetvalue (res, i, LOGIN_ID_POS)) + 1);
         strcpy (buffer[2], PQgetvalue (res, i, LOGIN_ID_POS));
         buffer[3] = (char *)g_malloc (strlen (get_access (res, i)) + 1);
         strcpy (buffer[3], get_access (res, i));
         buffer[4] = (char *)g_malloc (strlen (get_active_yn (res, i)) + 1);
         strcpy (buffer[4], get_active_yn (res, i));
         
         gtk_clist_append (lb_widget, buffer);
         g_free (buffer[0]);
         g_free (buffer[1]);
         g_free (buffer[2]);
         g_free (buffer[3]);
         g_free (buffer[4]);
         }
      rtn_val = TRUE;
      }
   
   PQclear (res);
   return rtn_val;
}

/*--------------------------------------------------------------------------
 * Signal handlers for the person list dialog.
 *--------------------------------------------------------------------------
 */
static void 
on_person_dlg_show (GtkWidget  *widget, gpointer   user_data)
{
   GtkToggleButton  *cb;
   GString          *config_str;
   gboolean          is_def;
   gboolean          flag;

   config_str = g_string_new ("");

   cb = GTK_TOGGLE_BUTTON (get_widget (widget, SHOW_INACTIVE_CB));
   g_assert (cb != NULL);
   g_string_sprintf (config_str, CONF_WITH_DEF, PACKAGE, DLG_INI_FILE,
                     USER_DLG_SEC, SHOW_INACT, "false");
   flag = gnome_config_get_bool_with_default (config_str->str, &is_def);
   gtk_toggle_button_set_active (cb, flag);

   g_string_free (config_str, TRUE);

   if (!draw_person_list (GTK_WINDOW (widget), user_data))
      {
      gtk_widget_destroy (widget);
      }
}


static void 
on_person_dlg_destroy (GtkObject  *object, gpointer   user_data)
{
   GList            *child_list;
   GList            *list_iter;
   GtkToggleButton  *cb;
   GString          *config_str;

   config_str = g_string_new ("");

   cb = GTK_TOGGLE_BUTTON (get_widget (GTK_WIDGET (object), SHOW_INACTIVE_CB));
   g_assert (cb != NULL);
   g_string_sprintf (config_str, BASIC_CONF, PACKAGE, DLG_INI_FILE,
                     USER_DLG_SEC, SHOW_INACT);
   gnome_config_set_bool (config_str->str, gtk_toggle_button_get_active (cb));

   g_string_sprintf (config_str, CONF_FILE, PACKAGE, DLG_INI_FILE);
   gnome_config_sync_file (config_str->str);

   g_string_free (config_str, TRUE);

   child_list = gtk_object_get_data (object, my_child_list);
   list_iter = g_list_first (child_list);
   while (list_iter)
      {
      gtk_signal_handler_block_by_func (GTK_OBJECT (list_iter->data),
                                        child_dead,
                                        (gpointer)my_child_list);
      gtk_widget_destroy (GTK_WIDGET (list_iter->data));
      list_iter = list_iter->next;
      }

   g_list_free (child_list);
}


static void
on_person_dlg_clicked (GnomeDialog *dlg,
                       gint         btn,
                       gpointer     user_data)
{
   save_dlg_size (GTK_WIDGET (dlg), USER_DLG_SEC, FALSE);
}


static void 
on_add_user (GtkWidget  *widget, gpointer   user_data)
{
   GtkWidget  *dlg;
   PGconn     *conn;
   gboolean    admin_user;
   GList      *child_list;
   
   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn);
   admin_user = (GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (user_data),
                               ADMIN_USER)) == 1);

   dlg = create_person_edit_dlg (FALSE, "", conn, 
                                 GTK_WIDGET (user_data), admin_user);
   gtk_widget_show (dlg);

   /*
    * Setup the child_dead() callback to remove the child from the
    * child_list when it dies.
    */
   gtk_signal_connect (GTK_OBJECT (dlg), "destroy",
                       GTK_SIGNAL_FUNC (child_dead),
                       (gpointer)my_child_list);

   child_list = gtk_object_get_data (GTK_OBJECT (user_data), my_child_list);
   child_list = g_list_append (child_list, dlg);
   gtk_object_set_data (GTK_OBJECT (user_data), my_child_list, child_list);
}


static void
raise_or_create_person_edit_dlg (GtkWidget   *parent,
                                 gboolean     is_update,
                                 gchar       *login_id,
                                 PGconn      *conn,
                                 gboolean     user_is_admin)
{
   GtkWidget  *dlg;
   GList      *child_list;
   GList      *list_iter;
   gchar      *tmp_login;

   /*
    * Go through the existing list of children first.  If we have
    * a child that already uses this login_id, just bring that one 
    * to the top.
    */
   child_list = gtk_object_get_data (GTK_OBJECT (parent), my_child_list);

   list_iter = g_list_first (child_list);
   while (list_iter)
      {
      /*
       * If we find a child displaying the same login_id, just raise it
       * and return.
       */
      tmp_login = gtk_object_get_data (GTK_OBJECT (list_iter->data),
                                       LOGIN_ID);
      if (tmp_login != NULL)
         {
         if (strcmp (tmp_login, login_id) == 0)
            {
            gdk_window_show (GTK_WIDGET (list_iter->data)->window);
            gdk_window_raise (GTK_WIDGET (list_iter->data)->window);

            return;
            }
         }
      list_iter = list_iter->next;
      }

   /*
    * Didn't find a child already displaying the user, so create one.
    */
   dlg = create_person_edit_dlg (is_update, login_id, conn, parent, 
                                 user_is_admin);
   gtk_widget_show (dlg);

   /*
    * Setup the child_dead() callback to remove the child from the
    * child_list when it dies.
    */
   gtk_signal_connect (GTK_OBJECT (dlg), "destroy",
                       GTK_SIGNAL_FUNC (child_dead),
                       (gpointer)my_child_list);

   child_list = g_list_append (child_list, dlg);
   gtk_object_set_data (GTK_OBJECT (parent), my_child_list, child_list);
}
                                


static void 
on_modify_user (GtkWidget  *widget, gpointer  user_data)
{
   GtkCList    *lb_widget;
   gint         row_idx;
   char        *login_id;
   PGconn      *conn;
   gboolean     admin_user;
   GtkWidget   *msg_dlg;
   
   lb_widget = GTK_CLIST (get_widget (GTK_WIDGET (user_data), PERSON_LIST));
   g_assert (lb_widget != NULL);
   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn);
   admin_user = (GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (user_data),
                               ADMIN_USER)) == 1);
   
   if (lb_widget->selection)
      {
      row_idx = GPOINTER_TO_INT (lb_widget->selection->data);
      gtk_clist_get_text (lb_widget, row_idx, LOGIN_ID_COL, &login_id);
      raise_or_create_person_edit_dlg (GTK_WIDGET (user_data), TRUE, 
                                       login_id, conn, admin_user);
      }
   else
      {
      msg_dlg = gnome_message_box_new
        (_("To modify a user, you must select a user to modify"),
         GNOME_MESSAGE_BOX_INFO,
         GNOME_STOCK_BUTTON_OK,
         NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      }
}


static void
on_person_sel_row (GtkWidget      *widget,
                   gint            row,
                   gint            col,
                   GdkEventButton *event,
                   gpointer        user_data)
{
   GtkCList   *clist = GTK_CLIST (widget);
   PGconn     *conn;
   gchar      *login_id;
   gboolean    admin_user;

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn);
   admin_user = (GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (user_data),
                               ADMIN_USER)) == 1);

   if (event)
      {
      switch (event->type)
         {
         case GDK_2BUTTON_PRESS:
            gtk_clist_get_text (clist, row, LOGIN_ID_COL, &login_id);
            raise_or_create_person_edit_dlg (GTK_WIDGET (user_data), TRUE,
                                             login_id, conn, admin_user);
            break;

         default:
            break;
         }
      }
}

static void
on_delete_user (GtkButton  *button, gpointer  user_data)
{
   GtkCList    *lb_widget;
   PGresult    *res;
   gint         row_idx;
   char        *login_id;
   PGconn      *conn;
   GtkWidget   *msg_dlg;

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   lb_widget = GTK_CLIST (get_widget (GTK_WIDGET (user_data), PERSON_LIST));
   g_assert (lb_widget != NULL);

   if (lb_widget->selection)
      {
      msg_dlg = gnome_message_box_new
         (_("Deleting a person is not reversable.\nAre you sure you want to continue?"),
          GNOME_MESSAGE_BOX_QUESTION,
          GNOME_STOCK_BUTTON_YES,
          GNOME_STOCK_BUTTON_NO,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 1);
      if (gnome_dialog_run (GNOME_DIALOG (msg_dlg)) == 0)
         {
         START_TRANSACTION (conn);
         row_idx = GPOINTER_TO_INT (lb_widget->selection->data);
         gtk_clist_get_text (lb_widget, row_idx, LOGIN_ID_COL, &login_id);
         res = delete_person (conn, login_id);
         if (chk_sql_error (res, _("Person Delete Failed")))
            {
            COMMIT (conn);
            }
         else
            {
            ROLLBACK (conn);
            }
         draw_person_list (GTK_WINDOW (user_data), conn);
         PQclear (res);
         }
      }
   else
      {
      msg_dlg = gnome_message_box_new
         (_("To delete a user, you must select a user to delete."),
          GNOME_MESSAGE_BOX_INFO,
          GNOME_STOCK_BUTTON_OK,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      }
}


static void
on_show_inactive_cb_toggled (GtkToggleButton *togglebutton,
                             gpointer         user_data)
{
   PGconn    *conn;

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);

   draw_person_list (GTK_WINDOW (user_data), conn);
}


static gboolean
on_button_press_event (GtkWidget      *widget,
                       GdkEventButton *event,
                       gpointer        user_data)
{
   gint        row;
   gint        col;
   gint        val;
   GtkWidget  *popup;
   GtkWidget  *main_win = GTK_WIDGET (user_data);
   gboolean    admin_user;

   static GnomeUIInfo popup_menu[] = 
      {
      GNOMEUIINFO_MENU_NEW_ITEM (N_("_New User"),
                                 N_("Create a new PRepS user."),
                                 on_add_user, NULL),
      GNOMEUIINFO_ITEM_NONE (N_("View/_Modify User"),
                             N_("View or Modify user's information."),
                             on_modify_user),
      GNOMEUIINFO_ITEM_NONE (N_("Delete User"),
                             N_("Delete the selected user."),
                             on_delete_user),
      GNOMEUIINFO_END
      };

   admin_user = (GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (main_win),
                                                       ADMIN_USER)) == 1);
   if (event)
      {
      switch (event->button)
         {
         case 1:
         case 2:
            break;
         case 3:
            switch (event->type)
               {
               case GDK_BUTTON_PRESS:
                  val = gtk_clist_get_selection_info (GTK_CLIST (widget),
                                                      event->x,
                                                      event->y,
                                                      &row,
                                                      &col);
                  popup = gnome_popup_menu_new (popup_menu);
                  if (val)
                     {
                     gtk_clist_select_row (GTK_CLIST (widget), row, col);
                     }
                  else
                     {
                     gtk_widget_set_sensitive (popup_menu[1].widget,
                                               FALSE);
                     gtk_widget_set_sensitive (popup_menu[2].widget,
                                               FALSE);
                     if (GTK_CLIST (widget)->selection != NULL)
                        {
                        gtk_clist_unselect_row
                           (GTK_CLIST (widget),
                            GPOINTER_TO_INT
                            (GTK_CLIST (widget)->selection->data), 0);
                        }
                     }
                  if (!admin_user)
                     {
                     gtk_widget_set_sensitive (popup_menu[0].widget,
                                               FALSE);
                     gtk_widget_set_sensitive (popup_menu[2].widget,
                                               FALSE);
                     }
                  gnome_popup_menu_do_popup_modal (popup,
                                                   NULL,
                                                   NULL,
                                                   event,
                                                   main_win);
                  gtk_widget_destroy (popup);
                  break;
               default:
                  break;
               }
         default:
            break;
         }
      }

   return FALSE;
}


static GtkWidget* 
create_person_dlg (GtkWidget *parent, PGconn *conn, gboolean user_is_admin)
{
  GtkWidget       *person_dlg;
  GtkWidget       *dialog_vbox11;
  GtkWidget       *vbox23;
  GtkWidget       *scrolledwindow4;
  GtkWidget       *person_clist;
  GtkWidget       *label1;
  GtkWidget       *label2;
  GtkWidget       *label3;
  GtkWidget       *label4;
  GtkWidget       *label5;
  GtkWidget       *hbox27;
  GtkWidget       *add_button;
  GtkWidget       *modify_button;
  GtkWidget       *delete_button;
  GtkWidget       *show_inactive_cb;
  GtkTooltips     *tooltips;

  tooltips = gtk_object_get_data (GTK_OBJECT (parent), TOOLTIPS);
  g_assert (tooltips != NULL);

  person_dlg = gnome_dialog_new (_("User List"),
                                 GNOME_STOCK_BUTTON_CLOSE,
                                 NULL);
  gnome_dialog_set_parent (GNOME_DIALOG (person_dlg), GTK_WINDOW (parent));
  gtk_object_set_data (GTK_OBJECT (person_dlg), PERSON_DIALOG, person_dlg);
  gtk_container_border_width (GTK_CONTAINER (person_dlg), 5);
  gtk_signal_connect (GTK_OBJECT (person_dlg), "show",
                      GTK_SIGNAL_FUNC (on_person_dlg_show),
                      conn);
  gtk_window_set_policy (GTK_WINDOW (person_dlg), FALSE, TRUE, FALSE);
  gtk_signal_connect (GTK_OBJECT (person_dlg), "destroy",
                      GTK_SIGNAL_FUNC (on_person_dlg_destroy),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (person_dlg), "clicked",
                      GTK_SIGNAL_FUNC (on_person_dlg_clicked),
                      NULL);
  gnome_dialog_set_close (GNOME_DIALOG (person_dlg), TRUE);
  gnome_dialog_set_default (GNOME_DIALOG (person_dlg), 0);
  set_dlg_size (person_dlg, USER_DLG_SEC, 590, 265);

  gtk_object_set_data (GTK_OBJECT (person_dlg), TOP_DOG, person_dlg);
  gtk_object_set_data (GTK_OBJECT (person_dlg), MY_DB_CONN, conn);
  gtk_object_set_data (GTK_OBJECT (person_dlg), ADMIN_USER,
        GINT_TO_POINTER (user_is_admin ? 1 : 0));

  dialog_vbox11 = GNOME_DIALOG (person_dlg)->vbox;
  gtk_box_set_spacing (GTK_BOX (dialog_vbox11), 5);
  gtk_object_set_data (GTK_OBJECT (person_dlg), 
                       "dialog_vbox11", 
                       dialog_vbox11);
  gtk_widget_show (dialog_vbox11);

  vbox23 = gtk_vbox_new (FALSE, 5);
  gtk_object_set_data (GTK_OBJECT (person_dlg), "vbox23", vbox23);
  gtk_widget_show (vbox23);
  gtk_box_pack_start (GTK_BOX (dialog_vbox11), vbox23, TRUE, TRUE, 0);

  scrolledwindow4 = gtk_scrolled_window_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (person_dlg), 
                       "scrolledwindow4", 
                       scrolledwindow4);
  gtk_widget_show (scrolledwindow4);
  gtk_box_pack_start (GTK_BOX (vbox23), scrolledwindow4, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow4), 
                                  GTK_POLICY_AUTOMATIC, 
                                  GTK_POLICY_AUTOMATIC);

  person_clist = gtk_clist_new (5);
  gtk_object_set_data (GTK_OBJECT (person_dlg), PERSON_LIST, person_clist);
  gtk_widget_show (person_clist);
  gtk_container_add (GTK_CONTAINER (scrolledwindow4), person_clist);
  gtk_clist_set_column_width (GTK_CLIST (person_clist), 0, 180);
  gtk_clist_set_column_width (GTK_CLIST (person_clist), 1, 130);
  gtk_clist_set_column_width (GTK_CLIST (person_clist), 2, 95);
  gtk_clist_set_column_width (GTK_CLIST (person_clist), 3, 65);
  gtk_clist_set_column_width (GTK_CLIST (person_clist), 4, 40);
  gtk_clist_column_titles_show (GTK_CLIST (person_clist));
  gtk_signal_connect (GTK_OBJECT (person_clist), "select_row",
                      GTK_SIGNAL_FUNC (on_person_sel_row),
                      person_dlg);
  gtk_signal_connect (GTK_OBJECT (person_clist), "button_press_event",
                      GTK_SIGNAL_FUNC (on_button_press_event),
                      person_dlg);

  label1 = gtk_label_new (_("Last Name"));
  gtk_object_set_data (GTK_OBJECT (person_dlg), "label1", label1);
  gtk_widget_show (label1);
  gtk_clist_set_column_widget (GTK_CLIST (person_clist), 0, label1);

  label2 = gtk_label_new (_("First Name"));
  gtk_object_set_data (GTK_OBJECT (person_dlg), "label2", label2);
  gtk_widget_show (label2);
  gtk_clist_set_column_widget (GTK_CLIST (person_clist), 1, label2);

  label3 = gtk_label_new (_("Login ID"));
  gtk_object_set_data (GTK_OBJECT (person_dlg), "label3", label3);
  gtk_widget_show (label3);
  gtk_clist_set_column_widget (GTK_CLIST (person_clist), 2, label3);

  label4 = gtk_label_new (_("Access"));
  gtk_object_set_data (GTK_OBJECT (person_dlg), "label4", label4);
  gtk_widget_show (label4);
  gtk_clist_set_column_widget (GTK_CLIST (person_clist), 3, label4);

  label5 = gtk_label_new (_("Active"));
  gtk_object_set_data (GTK_OBJECT (person_dlg), "label5", label5);
  gtk_widget_show (label5);
  gtk_clist_set_column_widget (GTK_CLIST (person_clist), 4, label5);

  hbox27 = gtk_hbox_new (FALSE, 5);
  gtk_object_set_data (GTK_OBJECT (person_dlg), "hbox27", hbox27);
  gtk_widget_show (hbox27);
  gtk_box_pack_start (GTK_BOX (vbox23), hbox27, FALSE, FALSE, 0);

  add_button = gtk_button_new_with_label (_("Add"));
  gtk_object_set_data (GTK_OBJECT (person_dlg), ADD_BUTTON, add_button);
  gtk_widget_show (add_button);
  gtk_box_pack_start (GTK_BOX (hbox27), add_button, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, add_button, _("Add a person."), NULL);
  gtk_signal_connect (GTK_OBJECT (add_button), "clicked",
                      GTK_SIGNAL_FUNC (on_add_user),
                      person_dlg);
  gtk_widget_set_sensitive (add_button, user_is_admin);

  modify_button = gtk_button_new_with_label (_("View/Modify"));
  gtk_object_set_data (GTK_OBJECT (person_dlg), 
                       MOD_BUTTON, 
                       modify_button);
  gtk_widget_show (modify_button);
  gtk_box_pack_start (GTK_BOX (hbox27), modify_button, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, 
                        modify_button, 
                        _("Modify an existing person."), 
                        NULL);
  gtk_signal_connect (GTK_OBJECT (modify_button), "clicked",
                      GTK_SIGNAL_FUNC (on_modify_user),
                      person_dlg);

  delete_button = gtk_button_new_with_label (_("Delete"));
  gtk_object_set_data (GTK_OBJECT (person_dlg), 
                       DEL_BUTTON, 
                       delete_button);
  gtk_widget_show (delete_button);
  gtk_box_pack_start (GTK_BOX (hbox27), delete_button, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, delete_button, _("Delete an existing person.  If the person is referenced in problem report or project, the user cannot be deleted, but can be made inactive."), NULL);
  gtk_signal_connect (GTK_OBJECT (delete_button), "clicked",
                      GTK_SIGNAL_FUNC (on_delete_user),
                      person_dlg);
  gtk_widget_set_sensitive (delete_button, user_is_admin);

  show_inactive_cb = gtk_check_button_new_with_label 
     (_("Show Inactive Persons"));
  gtk_object_set_data (GTK_OBJECT (person_dlg), 
                       SHOW_INACTIVE_CB, 
                       show_inactive_cb);
  gtk_widget_show (show_inactive_cb);
  gtk_box_pack_start (GTK_BOX (hbox27), show_inactive_cb, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (show_inactive_cb), "toggled",
                      GTK_SIGNAL_FUNC (on_show_inactive_cb_toggled),
                      person_dlg);

  gtk_object_set_data (GTK_OBJECT (person_dlg), TOOLTIPS, tooltips);

  return person_dlg;
}


void
destroy_person_dlg (void)
{
   if (person_dlg != NULL)
      {
      gtk_widget_destroy (person_dlg);
      person_dlg = NULL;
      }
}


void
show_person_dlg (GtkWidget *parent, PGconn *conn, gboolean user_is_admin)
{
   /*
    * If the dialog box exists, just make sure it is visable, otherwise
    * create it.
    */
   if (person_dlg != NULL)
      {
      gdk_window_show (person_dlg->window);
      gdk_window_raise (person_dlg->window);
      }
   else
      {
      person_dlg = create_person_dlg (parent, conn, user_is_admin);

      /*
       * When the dialog is destroyed, call gtk_widget_destroyed(),
       * which will set the "user_data" (&person_dlg in this case) to
       * NULL.
       */
      gtk_signal_connect (GTK_OBJECT (person_dlg),
                          "destroy",
                          GTK_SIGNAL_FUNC (gtk_widget_destroyed),
                          &person_dlg);
      gtk_widget_show (person_dlg);
      }
}


gboolean
person_dlg_dirty (void)
{
   if (person_dlg == NULL)
      {
      return FALSE;
      }

   return list_dlg_has_dirty_child (person_dlg, my_child_list);
}
